@@ -1,43 +1,48 @@
 import argparse
 import os
+import sys
 
 from cosmpy.aerial.client import LedgerClient
 from cosmpy.aerial.faucet import FaucetApi
 from cosmpy.aerial.wallet import LocalWallet
 from cosmpy.crypto.keypairs import PrivateKey
 from ptpython import embed
-from jenesis.config import Config
+from jenesis.config import Config, Profile
 from jenesis.contracts.detect import detect_contracts
-from jenesis.contracts.monkey import MonkeyContract
+from jenesis.contracts.monkey import make_contract
 from jenesis.contracts.observer import DeploymentUpdater
 from jenesis.keyring import query_keychain_items, query_keychain_item
-from jenesis.network import run_local_node
+from jenesis.network import network_context
 
 
-def load_config(args: argparse.Namespace) -> dict:
-    project_path = os.getcwd()
+PROJECT_PATH = os.getcwd()
+
+
+def get_profile(cfg: Config, args: argparse.Namespace) -> Profile:
 
     # check that we are actually running the command from the project root
-    if not os.path.exists(os.path.join(project_path, "jenesis.toml")):
-        # pylint: disable=all
+    if not os.path.exists(os.path.join(PROJECT_PATH, "jenesis.toml")):
         raise RuntimeError("Please run command from project root or create project first")
 
-    cfg = Config.load(project_path)
-    contracts = detect_contracts(project_path)
+    if args.profile is not None and args.profile not in cfg.profiles:
+        print(f'Invalid profile name. Expected one of {",".join(cfg.profiles.keys())}')
+        sys.exit(1)
+
+    profile_name = args.profile or cfg.get_default_profile()
+
+    return cfg.profiles.get(profile_name)
+
+
+def load_shell_globals(cfg: Config, selected_profile: Profile) -> dict:
 
     shell_globals = {}
     contract_instances = {}
 
-    if args.profile is not None and args.profile not in cfg.profiles:
-        print(f'Invalid profile name. Expected one of {",".join(cfg.profiles.keys())}')
-        return
+    contracts = {contract.name: contract for contract in detect_contracts(PROJECT_PATH)}
 
-    profile_name = args.profile or cfg.get_default_profile()
+    deployments = selected_profile.deployments
 
-    selected_profile = cfg.profiles.get(profile_name)
     if selected_profile is not None:
-        if selected_profile.network.is_local:
-            run_local_node(selected_profile.network)
 
         # build the ledger client
         client = LedgerClient(selected_profile.network)
@@ -46,41 +51,32 @@ def load_config(args: argparse.Namespace) -> dict:
 
         print('Detecting contracts...')
 
-        for contract in contracts:
-            print("C", contract)
+        for (deployment_name, deployment) in deployments.items():
+            contract = contracts[deployment.contract]
+            print("C", deployment_name)
 
             # skip contracts that we have not compiled
             if contract.digest() is None:
                 continue
 
-            # select the metadata for the contract
-            selected_contract = selected_profile.deployments.get(contract.name)
-
-            if selected_contract is not None:
-                address = selected_contract.address
-                code_id = selected_contract.code_id
-            else:
-                code_id = None
-                address = None
-
-            monkey = MonkeyContract(
+            monkey = make_contract(
                 contract,
                 client,
-                address=address,
-                code_id=code_id,
+                address=deployment.address,
+                code_id=deployment.code_id,
                 observer=DeploymentUpdater(
                     cfg,
-                    project_path,
-                    profile_name,
-                    contract.name,
+                    PROJECT_PATH,
+                    selected_profile.name,
+                    deployment_name,
                 ),
-                init_args=selected_contract.init,
+                init_args=deployment.init,
             )
 
-            contract_instances[contract.name] = monkey
+            contract_instances[deployment_name] = monkey
 
         print('Detecting contracts...complete')
-        
+
         shell_globals["ledger"] = LedgerClient(selected_profile.network)
         if selected_profile.network.faucet_url is not None:
             shell_globals["faucet"] = FaucetApi(selected_profile.network)
@@ -88,21 +84,21 @@ def load_config(args: argparse.Namespace) -> dict:
     wallets = {}
     print("Detecting local wallet keys...")
     try:
-        for key in query_keychain_items():
+        for key in query_keychain_items(cfg.keyring_backend):
             try:
-                info = query_keychain_item(key)
+                info = query_keychain_item(key, cfg.keyring_backend)
                 wallets[key] = LocalWallet(PrivateKey(info.private_key))
                 print(f"Importing wallets['{key}']")
             except Exception:
                 print(f"Failed to import local key '{key}'")
         print("Detecting local wallet keys...complete")
-    except Exception:
-        print("No local keychain found")
+    except Exception as ex:
+        print(f"No local keychain found: {ex}")
 
     shell_globals["cfg"] = cfg
-    shell_globals["project_path"] = project_path
-    shell_globals["profile"] = profile_name
-    shell_globals["contracts"] = {contract.name: contract for contract in contracts}
+    shell_globals["project_path"] = PROJECT_PATH
+    shell_globals["profile"] = selected_profile.name
+    shell_globals["contracts"] = contracts
     shell_globals["wallets"] = wallets
     for (name, instance) in contract_instances.items():
         shell_globals[name] = instance
@@ -111,9 +107,13 @@ def load_config(args: argparse.Namespace) -> dict:
 
 
 def run(args: argparse.Namespace):
-    shell_globals = load_config(args)
-    shell_globals.update(globals())
-    embed(shell_globals, vi_mode=False, history_filename=".shell_history")
+    cfg = Config.load(PROJECT_PATH)
+    profile: Profile = get_profile(cfg, args)
+    shell_globals = globals()
+
+    with network_context(profile.network, cfg.project_name, profile.name):
+        shell_globals.update(load_shell_globals(cfg, profile))
+        embed(shell_globals, vi_mode=False, history_filename=".shell_history")
 
 
 def add_shell_command(parser):
