@@ -13,18 +13,22 @@ from cosmpy.crypto.address import Address
 from jenesis.config.errors import ConfigurationError
 from jenesis.config.extract import (extract_opt_dict, extract_opt_int,
                                     extract_opt_str, extract_req_dict,
-                                    extract_req_str, extract_req_str_list)
+                                    extract_req_str, extract_req_str_list,
+                                    extract_opt_list)
 from jenesis.contracts import Contract
-from jenesis.contracts.detect import detect_contracts
+from jenesis.contracts.detect import detect_contracts, parse_contract
 from jenesis.network import (Network, fetchai_localnode_config,
-                             fetchai_testnet_config)
+                             fetchai_testnet_config, fetchai_mainnet_config)
+
 
 TEMPLATE_GIT_URL = "https://github.com/fetchai/jenesis-templates.git"
+DEFAULT_KEYRING_BACKEND = "os"
 
 
 @dataclass
 class Deployment:
-    contract: Contract  # internal: the contract that is deployed
+    name: str # internal: the name for the deployment
+    contract: str  # internal: the name of the contract to deploy
     network: str  # internal: the name of the network to deploy to
     deployer_key: str  # config: the name of the key to use for deployment
     init: Any  # config: init parameters for the contract
@@ -61,7 +65,7 @@ class Deployment:
         return hasher.hexdigest()
 
     def __repr__(self) -> str:
-        return f'{self.contract["contract"]}: {self.address}'
+        return f'{self.contract}: {self.address}'
 
     def is_configuration_out_of_date(self) -> bool:
         return self.checksum != self.compute_checksum()
@@ -85,7 +89,6 @@ class Deployment:
 class Profile:
     name: str
     network: Network
-    contracts: Dict[str, Contract]
     deployments: Dict[str, Deployment]
     default: bool = False
 
@@ -101,11 +104,12 @@ class Config:
     project_name: str
     project_authors: List[str]
     profiles: Dict[str, Profile]
+    keyring_backend: str = "os"
 
     def update_deployment(
         self,
         profile_name: str,
-        contract_name: str,
+        deployment_name: str,
         digest: str,
         code_id: int,
         address: Address,
@@ -114,14 +118,8 @@ class Config:
         if profile is None:
             raise ConfigurationError(f"unable to lookup profile {profile_name}")
 
-        contract = profile.contracts.get(contract_name)
-        deployment = profile.deployments.get(contract_name)
-        if deployment is None:
-            deployment = Deployment(
-                contract,
-                profile.network.name,
-                "", "", None, None, None, None, None
-            )
+        deployment = profile.deployments.get(deployment_name)
+        assert deployment is not None, f"Deployment not found: {deployment_name}"
 
         # update the contract if necessary
         if digest is not None:
@@ -131,7 +129,7 @@ class Config:
         if address is not None:
             deployment.address = Address(address)
 
-        profile.deployments[contract_name] = deployment
+        profile.deployments[deployment_name] = deployment
         self.profiles[profile_name] = profile
 
     def get_default_profile(self) -> str:
@@ -140,20 +138,28 @@ class Config:
                 return name
         return self.profiles.keys()[0]
 
+    def get_profile(self, profile_name: Optional[str]) -> Optional[Profile]:
+        if profile_name is None:
+            profile_name = self.get_default_profile()
+        else:
+            if profile_name not in self.profiles:
+                print(f'Invalid profile name. Expected one of {",".join(self.profiles.keys())}')
+                return None
+        return self.profiles[profile_name]
+
     @classmethod
     def load(cls, path: str) -> "Config":
         project_file_path = os.path.join(path, "jenesis.toml")
         lock_file_path = os.path.join(path, "jenesis.lock")
 
         if not os.path.isfile(project_file_path):
-            raise ConfigurationError('Missing project file: "jenesis.lock"')
+            raise ConfigurationError('Missing project file: "jenesis.toml"')
         project_contents = toml.load(project_file_path)
 
         if os.path.isfile(lock_file_path):
             lock_file_contents = toml.load(lock_file_path)
         else:
             lock_file_contents = {}
-            print("No lock file found. Proceeding without any contract deployments.")
 
         return cls._loads(project_contents, lock_file_contents)
 
@@ -176,10 +182,15 @@ class Config:
             profile = cls._parse_profile(name, config_item, lock_item)
             profiles[profile.name] = profile
 
+        keyring_backend = extract_opt_str(
+            project_contents, "project.keyring_backend"
+        ) or DEFAULT_KEYRING_BACKEND
+
         return Config(
             project_name=extract_req_str(project_contents, "project.name"),
             project_authors=extract_req_str_list(project_contents, "project.authors"),
             profiles=profiles,
+            keyring_backend=keyring_backend,
         )
 
     @classmethod
@@ -201,13 +212,13 @@ class Config:
 
         deployments = {}
         if "contracts" in profile:
-            for contract_name, contract_settings in profile_contracts.items():
-                deployment_lock = lock_profile.get(contract_name, {})
+            for deployment_name, contract_cfg in profile_contracts.items():
+                deployment_lock = lock_profile.get(deployment_name, {})
 
                 deployment = cls._parse_contract_config(
-                    contract_settings, network.name, deployment_lock
+                    deployment_name, contract_cfg, network.name, deployment_lock
                 )
-                deployments[contract_name] = deployment
+                deployments[deployment_name] = deployment
 
         is_default = False
         if "default" in profile:
@@ -217,16 +228,15 @@ class Config:
         return Profile(
             name=str(name),
             network=network,
-            contracts=profile_contracts,
             deployments=deployments,
             default=is_default,
         )
 
     @classmethod
     def _parse_contract_config(
-        cls, contract: dict, network: str, lock: Any
+        cls, deployment_name: str, contract_cfg: dict, network: str, lock: Any
     ) -> Deployment:
-        if not isinstance(contract, dict):
+        if not isinstance(contract_cfg, dict):
             raise ConfigurationError(
                 "contract configuration invalid, expected dictionary"
             )
@@ -239,11 +249,12 @@ class Config:
             return None if value is None else Address(value)
 
         return Deployment(
-            contract=contract,
+            name=str(deployment_name),
+            contract=extract_req_str(contract_cfg, "contract"),
             network=str(network),
-            init=extract_opt_dict(contract, "init"),
-            deployer_key=extract_req_str(contract, "deployer_key"),
-            init_funds=extract_opt_str(contract, "init_funds"),
+            init=extract_opt_dict(contract_cfg, "init"),
+            deployer_key=extract_req_str(contract_cfg, "deployer_key"),
+            init_funds=extract_opt_str(contract_cfg, "init_funds"),
             digest=extract_opt_str(lock, "digest"),
             address=opt_address(extract_opt_str(lock, "address")),
             code_id=extract_opt_int(lock, "code_id"),
@@ -274,17 +285,17 @@ class Config:
         # detect contract source code and add placeholders for key contract data
         contracts = detect_contracts(project_root) or []
 
-
-        contract_cfgs = {contract.name: Deployment(contract,
+        deployments = {contract.name: Deployment(contract.name, contract.name,
             network_name, "", {arg: "" for arg in contract.init_args()},
             "", None, None, None, None,
         ) for contract in contracts}
 
-
         if network_name == "fetchai-testnet":
             net_config = fetchai_testnet_config()
         elif network_name == "fetchai-localnode":
             net_config = fetchai_localnode_config()
+        elif network_name == "fetchai-mainnet":
+            net_config = fetchai_mainnet_config()
         else:
             raise ConfigurationError("Network name not recognized")
 
@@ -294,13 +305,13 @@ class Config:
         profiles = {
             profile: {
                 "network": network,
-                "contracts": {name: vars(cfg) for (name, cfg) in contract_cfgs.items()},
+                "contracts": {name: vars(cfg) for (name, cfg) in deployments.items()},
                 "default": True,
             }
         }
 
         data = {
-            "project": {"name": project_name, "authors": authors},
+            "project": {"name": project_name, "authors": authors, "keyring_backend": "os"},
             "profile": profiles,
         }
 
@@ -322,27 +333,25 @@ class Config:
                 pass
 
     @staticmethod
-    def update_project(path: str, profile: str, network_name: str, contract: Contract):
+    def update_project(path: str, profile: str, network_name: str, contract: Contract, deployment_name: str):
 
         # take the project name directly from the base name of the project
         project_root = os.path.abspath(path)
 
-
-        contract_cfg = Deployment(contract,
+        deployment = Deployment(deployment_name, contract.name,
             network_name, "", {arg: "" for arg in contract.init_args()},
             "", None, None, None, None)
 
-
         data = toml.load("jenesis.toml")
 
-        data["profile"][profile]["contracts"][contract.name] = vars(contract_cfg)
+        data["profile"][profile]["contracts"][deployment_name] = vars(deployment)
         project_configuration_file = os.path.join(project_root, "jenesis.toml")
 
         with open(project_configuration_file, "w", encoding="utf-8") as toml_file:
             toml.dump(data, toml_file)
 
     @staticmethod
-    def update_key(path: str, profile: str, contract: Contract, key: str):
+    def update_key(path: str, profile: str, deployment_name: str, key: str):
 
         # take the project name directly from the base name of the project
         project_root = os.path.abspath(path)
@@ -352,14 +361,14 @@ class Config:
         with open(path, encoding="utf-8") as toml_file:
             data = toml.load(toml_file)
 
-        data["profile"][profile]["contracts"][contract.name]["deployer_key"] = key
+        data["profile"][profile]["contracts"][deployment_name]["deployer_key"] = key
         project_configuration_file = os.path.join(project_root, "jenesis.toml")
 
         with open(project_configuration_file, "w", encoding="utf-8") as toml_file:
             toml.dump(data, toml_file)
 
     @staticmethod
-    def add_profile(path: str, profile: str, network_name: str):
+    def add_profile(profile: str, network_name: str):
 
         data = toml.load("jenesis.toml")
 
@@ -367,33 +376,24 @@ class Config:
             net_config = fetchai_testnet_config()
         elif network_name == "fetchai-localnode":
             net_config = fetchai_localnode_config()
+        elif network_name == "fetchai-mainnet":
+            net_config = fetchai_mainnet_config()
         else:
             raise ConfigurationError("Network name not recognized")
 
         network = {"name": ""}
         network.update(vars(net_config))
 
-        # detect contract source code and add placeholders for key contract data
-        contracts = detect_contracts(path) or []
+        default_profile = list(data["profile"].keys())[0]
 
-        contract_cfgs = {
-            contract.name: Deployment(
-                contract,
-                network_name,
-                "",
-                {arg: "" for arg in contract.init_args()},
-                "",
-                None,
-                None,
-                None,
-                None,
-            )
-            for contract in contracts
-        }
+        contract_data = data["profile"][default_profile]["contracts"]
+
+        for deployment_name in contract_data.keys():
+            contract_data[deployment_name]["network"] = network_name
 
         data["profile"][profile] = {
             "network": network,
-            "contracts": {name: vars(cfg) for (name, cfg) in contract_cfgs.items()},
+            "contracts": contract_data,
         }
 
         output_file_name = "jenesis.toml"
@@ -402,7 +402,9 @@ class Config:
             toml.dump(data, toml_file)
 
     @staticmethod
-    def add_contract(contract_path: str, template: str, name: str, branch: str):
+    def add_contract(project_root: str, template: str, name: str, branch: str) -> Contract:
+
+        contract_path = os.path.join(project_root, 'contracts', name)
 
         # create the temporary clone folder
         temp_clone_path = mkdtemp(prefix="jenesis-", suffix="-tmpl")
@@ -419,9 +421,10 @@ class Config:
             )
 
         # find the target contract
+        available_templates = os.listdir(os.path.join(temp_clone_path, "contracts"))
         contract_template_path = os.path.join(temp_clone_path, "contracts", template)
         if not os.path.isdir(contract_template_path):
-            print(f"Unknown template {template}")
+            print(f"Unknown template {template}: expecting one of {set(available_templates)}")
             return
         print("Downloading template...complete")
 
@@ -446,3 +449,5 @@ class Config:
 
         # clean up the temporary folder
         shutil.rmtree(temp_clone_path)
+
+        return parse_contract(project_root, name)
