@@ -1,6 +1,6 @@
 from abc import ABC, abstractmethod
 from enum import Enum
-
+from typing import Any, List, Optional
 
 class TaskStatus(Enum):
     IDLE = 0
@@ -20,6 +20,11 @@ class Task(ABC):
     def status(self) -> TaskStatus:
         pass
 
+    @property
+    @abstractmethod
+    def logs_text(self) -> str:
+        pass
+
     @property
     @abstractmethod
     def status_text(self) -> str:
@@ -48,3 +53,7 @@ class Task(ABC):
     @abstractmethod
     def poll(self):
         pass
+
+    @abstractmethod
+    def teardown(self):
+        pass
